"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.spaceify = exports.l33t1fy = exports.smashify = exports.emojify = exports.applyFitzpatrick = exports.altCapify = exports.clapify = void 0;
const emojis_1 = require("./emojis");
const constants_1 = require("./constants");
const emojifyValues = Object.values(emojis_1.emojis).filter((x) => x.category !== "flags");
/**
 * Transforms text to be all caps with clap emojis between each word
 * @param text the text to be transformed
 * @returns the text with clap emojis
 */
function clapify(text, options) {
    const arr = text === null || text === void 0 ? void 0 : text.toUpperCase().split(/(\s+)/).filter((e) => e.trim().length > 0);
    const clapEmoji = applyFitzpatrick(emojis_1.emojis.clap, options === null || options === void 0 ? void 0 : options.fitzpatrick);
    return (arr === null || arr === void 0 ? void 0 : arr.length) ? arr.join(` ${clapEmoji} `) : "";
}
exports.clapify = clapify;
/**
 * Transforms text to have alternating capitalization
 * @param text the text to be transformed
 * @returns the text with alternating capitalization
 */
function altCapify(text) {
    const arr = text.split("");
    for (let i = 0; i < arr.length; i++) {
        arr[i] = i % 2 ? arr[i].toUpperCase() : arr[i].toLowerCase();
    }
    return arr.join("");
}
exports.altCapify = altCapify;
/**
 * Applies fitzpatrick modifier to an emoji
 * @param emoji the emoji to modify
 * @param modifier the fitzpatrick modifier
 * @returns the modified emoji character
 */
function applyFitzpatrick(emoji, modifier) {
    if (emoji.fitzpatrick_scale && modifier) {
        // move female modifier to end for proper unicode ordering
        // more info here: https://unicode.org/emoji/charts/emoji-zwj-sequences.html
        if (emoji.char.includes(constants_1.femaleModifier)) {
            return emoji.char.replace(constants_1.femaleModifier, '') + modifier + constants_1.femaleModifier;
        }
        // move male modifier to end for proper unicode ordering
        if (emoji.char.includes(constants_1.maleModifier)) {
            return emoji.char.replace(constants_1.maleModifier, '') + modifier + constants_1.maleModifier;
        }
        return emoji.char + modifier;
    }
    return emoji.char;
}
exports.applyFitzpatrick = applyFitzpatrick;
/**
 * Inserts emojis into text
 * @param text the source text
 * @returns the text with emojis
 */
function emojify(text, options) {
    const textArr = text.split(/(\s+)/).filter((e) => e.trim().length > 0);
    const newTextArr = [];
    textArr.forEach((x) => {
        newTextArr.push(x);
        const searchText = x.replace(/[^a-zA-Z ]/g, "").toLowerCase();
        // first, look for exact match
        const match = emojis_1.emojis[searchText];
        if (match) {
            newTextArr.push(applyFitzpatrick(match, options === null || options === void 0 ? void 0 : options.fitzpatrick));
        }
        else {
            // second, look for keywords
            const filteredEmojis = emojifyValues.filter((y) => { var _a; return (_a = y.keywords) === null || _a === void 0 ? void 0 : _a.includes(searchText); });
            if (filteredEmojis.length) {
                const keywordEmoji = filteredEmojis[Math.floor(Math.random() * filteredEmojis.length)];
                newTextArr.push(applyFitzpatrick(keywordEmoji, options === null || options === void 0 ? void 0 : options.fitzpatrick));
            }
            else {
                // third, randomly select among common emojis
                const emojiCount = Math.random() * 6 - 2;
                if (emojiCount > 0) {
                    const randomEmoji = constants_1.emojifyDefaults[Math.floor(Math.random() * constants_1.emojifyDefaults.length)];
                    const randomEmojiFitzpatrick = applyFitzpatrick(randomEmoji, options === null || options === void 0 ? void 0 : options.fitzpatrick);
                    newTextArr.push(randomEmojiFitzpatrick.repeat(emojiCount));
                }
            }
        }
    });
    return newTextArr.join(" ");
}
exports.emojify = emojify;
/**
 * Translates text into keyboard smashing
 * @param text the text to be translated
 * @returns the generated keysmash text
 */
function smashify(text) {
    const leftHandKeys = ["a", "s", "d", "f", "g"];
    const rightHandKeys = ["h", "j", "k", "l", ";"];
    const chars = [];
    const startingChars = ["a", "s", "d", "f"];
    let hand = leftHandKeys;
    for (let i = 0; i < text.length; i++) {
        // 75% chance to short-circtuit with common starting keys
        if (i < startingChars.length && Math.random() > 0.25) {
            chars.push(startingChars[i]);
            continue;
        }
        const c = text[i];
        if (c.trim()) {
            chars.push(hand[Math.floor(Math.random() * hand.length)]);
            if (hand === leftHandKeys) {
                hand = rightHandKeys;
            }
            else {
                hand = leftHandKeys;
            }
        }
        else {
            chars.push(c);
        }
    }
    return chars.join("");
}
exports.smashify = smashify;
/**
 * Translates the text into l33t sp34k
 * @param text the text to be translated
 * @param options options
 * @returns the translated text
 */
function l33t1fy(text, options) {
    let characterMap = {};
    let preserveCapitalization = false;
    if (!(options === null || options === void 0 ? void 0 : options.substitutionType)) {
        return "";
    }
    switch (options === null || options === void 0 ? void 0 : options.substitutionType) {
        case "numbers":
            characterMap = constants_1.numbers;
            break;
        case "big":
            characterMap = constants_1.bigLetters;
            break;
        case "symbols":
            characterMap = constants_1.symbols;
            break;
        case "homophones":
            characterMap = constants_1.homophones;
            preserveCapitalization = true;
            break;
    }
    const transformedText = [];
    text.split("").forEach((c) => {
        // TODO: preserve capitalization for homophones
        transformedText.push(characterMap[c.toLowerCase()] || c);
    });
    return transformedText.join("");
}
exports.l33t1fy = l33t1fy;
/**
 * Text transform for spaced out text
 * @param text the text to be transformed
 * @returns the spaced out text
 */
function spaceify(text) {
    return text.split("").join(" ");
}
exports.spaceify = spaceify;
